"
Newsgroups: comp.lang.smalltalk
From: al938@FreeNet.Carleton.CA (Steve Chepurny)
Subject: Re: Binary Trees - SmallTalk
Message-ID: <CzLvzp.Lnn@freenet.carleton.ca>
Sender: al938@freenet3.carleton.ca (Steve Chepurny)
Organization: The National Capital FreeNet, Ottawa, Ontario, Canada
Date: Mon, 21 Nov 1994 07:11:48 GMT
Lines: 363
"

"
Here's a complete implementation of a binary tree class:
"
 

Object subclass: #BinaryTree
  instanceVariableNames: 
    'data leftSubtree rightSubtree '
  classVariableNames: ''
  poolDictionaries: ''   !
 

!BinaryTree class methods !

empty
    "Returns a new empty binary tree with subtrees as binary trees"
    ^super new
	    leftSubtree: self nonEmpty;
	    rightSubtree: self nonEmpty!

new
    "Returns an error message"
   self error: 'use empty instead of new'!

nonEmpty
    "Returns a new binary tree with instances set"
    ^super new! !

!BinaryTree methods !

data
    ^data!

data:   anObject
    data := anObject!

depth
    "Returns the depth of the binary list"
	|answer|

	self isEmpty ifTrue: [^0].
	answer := (self level).
	^answer - 1.!

do: aBlock
    "applies aBlock to each element in the binary tree in inorder"

    self isEmpty ifTrue: [^self].
    self leftSubtree isNil
	ifFalse: [self leftSubtree do: aBlock].
    aBlock value: self data.
    self rightSubtree isNil
	ifFalse:  [self rightSubtree do: aBlock].!

getTreeWithAnInteger: anInteger
    "Private - Returns the BinaryTree with data anInteger.  If anInteger
      not in the tree it returns nil."

    self inOrderDo: [:each|each data = anInteger ifTrue:[^each]].
    ^nil.!

inOrderDo: aBlock
    "Traverses the elements of the binary tree in
	LEFT - ROOT - RIGHT order, applying a block
	to each element"

    self isEmpty ifTrue: [^self].
    self leftSubtree isNil
	ifFalse: [self leftSubtree inOrderDo: aBlock].
    aBlock value: self .
    self rightSubtree isNil
	ifFalse:  [self rightSubtree inOrderDo: aBlock].!

inOrderSuccessor
    "Private - Returns the in-order successor (a BST) of receiver.
	If receiver is empty then returns the receiver."

    self rightSubtree isEmpty ifTrue:[^self].
    self rightSubtree inOrderDo: [:each| ^each].!

isEmpty
    "returns true if the binary tree is empty and false otherwise"
    ^self data isNil!

isLeaf
    "Returns true if self is a leaf and false otherwise"
    ^((self leftSubtree isNil) and: [self rightSubtree isNil])!

leftSubtree
    ^leftSubtree!

leftSubtree: aBinaryTree
    leftSubtree := aBinaryTree!

level
    "Returns the depth of the binary list"
	self isEmpty ifTrue: [^0].
	^(1 + ((self leftSubtree level) max: (self rightSubtree level))) .!

postOrderDo: aBlock
    "Traverses the elements of the binary tree in
	LEFT - RIGHT - ROOT order, applying a block
	to each element"

    self isEmpty ifTrue: [^self].
    self leftSubtree isNil
	ifFalse: [self leftSubtree postOrderDo: aBlock].
    self rightSubtree isNil
	ifFalse:  [self rightSubtree postOrderDo: aBlock].
    aBlock value: self .!

preOrderDo: aBlock
    "Traverses the elements of the binary tree in
	ROOT - LEFT - RIGHT order, applying a block
	to each element"

    self isEmpty ifTrue: [^self].
    aBlock value: self .
    self leftSubtree isNil
	ifFalse: [self leftSubtree preOrderDo: aBlock].
    self rightSubtree isNil
	ifFalse:  [self rightSubtree preOrderDo: aBlock].!

printOn: aStream
    "Append the ascii representation to aStream"

    self isEmpty
	ifTrue: [aStream nextPutAll: '--']
	ifFalse: [
	    aStream nextPut: $(.
	    self data printOn: aStream. aStream nextPut: $ .
	    self leftSubtree printOn: aStream. aStream nextPut: $ .
	    self rightSubtree printOn: aStream.
	    aStream nextPut: $)]!

rightSubtree
    ^rightSubtree!

rightSubtree: aBinaryTree
    rightSubtree := aBinaryTree!

size
    "Returns the size of the binary tree by traversing each element inorder"

    |count|

    count := 0.
    self inOrderDo: [:each| count := count + 1].
    ^count! !
 

"
That's it. You should be able to file it right in. 


Steve
--
-----------------------------------------------------------------------------
Steve Chepurny                     |         Carleton University
al938@freenet.carleton.ca          |         Psychology II    
------------------------------------------------------------------------------
"
