"       NAME            CompressedOrderedCollection
	AUTHOR          RMD@cs.man.ac.uk
	FUNCTION like RunArrays, for OrderedCollections 
	ST-VERSIONS     2.2
	PREREQUISITES    
	CONFLICTS       
	DISTRIBUTION    world
	VERSION         1.1
	DATE    22 Jan 1989
SUMMARY CompressedOrderedCollection
	Similar to RunArrays, but for
	OrderedCollections. (2.2). RMD.
"!

Object subclass: #Run
	instanceVariableNames: 'length value '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Sequenceable'
!

Run comment:
'I represent a sequence of a particular value. I am used by CompressedOrderedCollection.


Instance Variables

length <Integer>        The number of instances of my value which I represent.
value   <Anything>      The value which I represent'!


!Run methodsFor: 'removing'!

decreaseLength: anInteger
	"Remove anInteger items, returning my value"

	length - anInteger < 0
	ifTrue:[self error: 'under run'].
	length := length - anInteger.
	^value!

decrementLength
	"Remove one item, returning my value"

	length <= 0
	ifTrue:[self error: 'under run'].
	length := length - 1.
	^value! !

!Run methodsFor: 'adding'!

increaseLength: anInteger
	"Add anInteger to the length."

	length := length + anInteger!

incrementLength
	"Add one to the length."

	length := length + 1! !

!Run methodsFor: 'accessing'!

length
	"Return the length of this run"

	^length!

length: anInteger
	"Set the length of this run"

	length := anInteger!

length: anInteger value: anObject
	"set the length and value of this run"

	length := anInteger.
	value := anObject!

value
	"Return the value of this run"

	^value!

value: anObject
	"Set the value of this run"

	value := anObject! !

!Run methodsFor: 'printing'!

printOn: aStream 
	"append to aStream a sequence of characters that identifies the 
	receiver. "

	aStream nextPutAll: 'a Run(length: ' , self length printString , ' value: '
 , self value printString , ')'!

storeOn: aStream 
	"Append to aStream an expression which, if evaluated, will generate  
	an object similar to the receiver."

	aStream nextPutAll: 'Run new length: ' , self length printString , ' value:
 ' , self value printString! !

!Run class methodsFor: 'instance creation'!

length: anInteger value: anObject
	"Create a run of length anInteger with value anObject."

	^super new length: anInteger value: anObject!

new
	"You probably shouldn't use this"

	self length: 0 value: nil
! !

OrderedCollection variableSubclass: #CompressedOrderedCollection
	instanceVariableNames: 'collection '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Sequenceable'
!

CompressedOrderedCollection comment:
'This implements an ordered collection which uses runs to minimise the amount
 of data that it holds. Basically it should be used if you know that an ordered
 collections is giong to contain a lot of runs of eactly the same data. Implemented
 to allow simultation playback, since the ordered collctions which that generates
 are so big that the complier falls over, though most of it is extremely repetetive.
 This should be totally abstracted. The user should not be a ble to see the difference
 between an ordered collection and a ComrpessedOrderedCollection.  This has a
 lot in common with RunArray, and the two should probably share implementation.
 but I could not do some of the things I wanted with the RunArray code, so this
 is all done on its own.
	Some of this could be made faster by adding a cache of the start and finish
 indices of each run, but since I envisage that most additions etc. will be to
 and from the end those are not included. In addition I have implemented the
 bare essentials of this for what I need it for - i.e. add to the end and take
 off the beginning.'!


!CompressedOrderedCollection methodsFor: 'accessing'!

after: oldObject 
	"Answer the element after oldObject.  If the receiver does not contain 
	 oldObject or if the receiver contains no elements after oldObject,  
	provide an error notification. This is a dumb thing to do to a 
	compressed collection, so no attempt has been made to make it 
	efficient. It answers with the element immediately after oldObject, 
	which probably means it is in the same run and will have the same 
	value. "

	| index |
	index := self find: oldObject.
	index = self size
		ifTrue: [^self errorLastObject]
		ifFalse: [^self at: index + 1]!

at: anInteger 
	"Answer the element at index anInteger. 
	at: is used by a knowledgeable client to access an existing element 
	This is a pretty dumb thing to do a compressed collection and it is 
	not at all efficient (think of that as a discouragement."

	| position |
	position := 1.
	(anInteger < 1 or: [anInteger > self size])
		ifTrue: [self errorNoSuchElement]
		ifFalse: [self do: [:element | (position = anInteger)
					ifTrue: [^element]
					ifFalse: [position := position + 1]]]!

at: anInteger put: anObject 
	"Put anObject at element index anInteger.      
	at:put: can not be used to append, front or back, to an ordered      
	collection;  it is used by a knowledgeable client to replace an     
	element. It doesn't make a lot of sense for a compressed collection,    
	and as you can see, the implementation is awful - still if you will    
	insist on using this what can you expect. Basically this just copies 
	itself up to the start point then adds the required element then 
	copies the rest of itself"

	| newCollection |
	(anInteger < 1 or: [anInteger > self size])
		ifTrue: ["first check the range"
			self errorNoSuchElement]
		ifFalse: [(anInteger = 1 or: [anInteger = self size])
				ifTrue: [self error: 'Cannot use this for first or last elements']
				ifFalse: 
					[newCollection := self copyFrom: 1 to: anInteger - 1.
					newCollection addLast: anObject.
					newCollection addAll: (self copyFrom: anInteger + 1 to: self size).
					self become: newCollection]]!

before: oldObject 
	"Answer the element before oldObject.  If the receiver does not 
	contain oldObject or if the receiver contains no elements before 
	oldObject, provide an error notification. This is a dumb thing to do to 
	a compressed collection, so no attempt has been made to make it 
	efficient. It answers with the element immediately before oldObject, 
	which probably means it is in the same run and will have the same 
	value. "

	| index |
	index := self find: oldObject.
	index = 1
		ifTrue: [^self errorFirstObject]
		ifFalse: [^self at: index - 1]!

first
	"Return the value of my first element."

	self emptyCheck.
	^collection first value!

last
	"Return the value of my last element."

	self emptyCheck.
	^collection last value!

size
	"Answer how many elements the receiver contains."

	| size |
	size := 0.
	collection do: [:run | size := size + run length].
	^size! !

!CompressedOrderedCollection methodsFor: 'adding'!

add: anObject beforeIndex: spot 
	"Add the argument, newObject, as an element of the receiver.  Put it
	in the position just preceding the indexed position spot.  Answer newObject."


	self insert: anObject before: spot!

addAll: aCollection
	"Add all elements of the argument, aCOllection, at the end of the receiver."


	aCollection do:[:element | self addLast:element]!

addFirst: newObject 
	"Add newObject to the start of this collection."
	"First see if there is anything in this collection - if not just insert a 
	new run."

	collection isEmpty
		ifTrue: [collection addLast: (Run length: 1 value: newObject)]
		ifFalse: ["If this is the same value as the first run add it to the run, 
			otherwise add a new run"
			collection first value = newObject
				ifTrue: [collection first incrementLength]
				ifFalse: [collection addFirst: (Run length: 1 value: newObject)]]!

add:newObject
	self addLast:newObject
!

addLast: newObject 
	"Add newObject to the end of this collection."
	"First see if there is anything in this collection - if not just insert a 
	new run."

	collection isEmpty
		ifTrue: [collection addLast: (Run length: 1 value: newObject)]
		ifFalse: ["If this is the same value as the last run add it to the run, 
			otherwise add a new run"
			collection last value = newObject
				ifTrue: [collection last incrementLength]
				ifFalse: [collection addLast: (Run length: 1 value: newObject)]]!

grow

self nogo! !

!CompressedOrderedCollection methodsFor: 'removing'!

remove: oldObject ifAbsent: anExceptionBlock 
	"Remove oldObject as one of the receiver's elements.  If several of the 
	elements are equal to oldObject, only one is removed. If no element is 
	equal to 
	oldObject, answer the result of evaluating anExceptionBlock.  
	Otherwise, answer the argument, oldObject."

	| index found |
	found := true.
	index := self find: oldObject ifAbsent: [found := false].
	found
		ifTrue: 
			[self removeIndex: index.
			^oldObject]
		ifFalse: [^anExceptionBlock value]!

removeAllSuchThat: aBlock 
	"Evaluate aBlock for each element of the receiver.  Remove each 
	element for 
	which aBlock evaluates to true.   
	This is easy - just check for the value of each run and if ablock is 
	true remove the run"

	collection removeAllSuchThat: [:aRun | aBlock value: aRun value]!

removeAtIndex: anIndex 
	"Remove the element of the collection at position anIndex.  
	Re-implement rather than inherit so that we don't have to faff 
	about with firstIndex whihc is not used anywhere else. Answer the 
	object removed."

	| obj |
	obj := self at: anIndex.
	self removeIndex: anIndex.
	^obj!

removeFirst
	"Remove the first element of the receiver.  If the receiver is empty,   
	provide an error notification."

	| returnValue run |
	self emptyCheck.
	run := collection first.
	returnValue := run decrementLength.
	run length = 0 ifTrue: [collection removeFirst].
	^returnValue!

removeLast
	"Remove the Last element of the receiver.  If the receiver is empty,   
	provide an error notification."

	| returnValue run |
	self emptyCheck.
	run := collection last.
	returnValue := run decrementLength.
	run length = 0 ifTrue: [collection removeLast].
	^returnValue! !

!CompressedOrderedCollection methodsFor: 'enumerating'!

do: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the 
	argument. "

	| runValue |
	collection do: 
		[:aRun | 
		runValue := aRun value.
		1 to: aRun length do: [:dummy | aBlock value: runValue]]!

reverseDo: aBlock
	"Evaluate aBlock with each of the receiver's elements as the argument, starting

	with the last element and taking each in sequence up to the first."

	| runValue |
	collection reverseDo: 
		[:aRun | 
		runValue := aRun value.
		aRun length timesRepeat: [aBlock value: runValue]]! !

!CompressedOrderedCollection methodsFor: 'user interface'!

inspect
	"Re implement so that they don't get an ordered collection inspector 
	which would get very confused."

	self basicInspect! !

!CompressedOrderedCollection methodsFor: 'private'!

addRun: aRun 
	"Appends aRun to this collection. This is a fast mechanism for 
	constructing compressed collections. Used by storeOn."

	collection addLast: aRun!

addRunWithLength: anInteger value: anObject
	"Creates the specified run and adds it to this collection. This is a fast mechanism
 for constructing compressed collections. Used by storeOn."

	self addRun: (Run new length: anInteger value: anObject)!

find: oldObject 
	"If I contain oldObject return its index, otherwise create an error   
	notifier. It will answer with the position of the very first element of  
	that value."

	| position |
	position := 1.
	self do: [:element | element = oldObject
			ifTrue: [^position]
			ifFalse: [position := position + 1]].
	self errorNoSuchElement!

find: oldObject ifAbsent: exceptionBlock 
	"If I contain oldObject return its index, otherwise execute the 
	exception block. It will answer with the position of the very first 
	element of that value."

	| position |
	position := 1.
	self do: [:element | element = oldObject
			ifTrue: [^position]
			ifFalse: [position := position + 1]].
	^exceptionBlock value!

insert: anObject before: anIndex 
	"Insert anObject before the object currently at position anIndex"

	| newCollection |
	newCollection := self copyFrom: 1 to: anIndex - 1.
	newCollection addLast: anObject.
	newCollection addAll: (self copyFrom: anIndex to: self size).
	self become: newCollection!

isEmpty
	"Am I empty or not. Returns a boolean"

	^collection isEmpty!

nogo
	self error: 'Does not make sense for a compressed collection : Not implemented.'!


removeIndex: removedIndex 
	"Remove the element of the collection at position anIndex."

	| newCollection |
	newCollection := self copyFrom: 1 to: removedIndex - 1.
	newCollection addAll: (self copyFrom: removedIndex + 1 to: self size).
	self become: newCollection! !

!CompressedOrderedCollection methodsFor: 'initialising'!

initialize
	"To initialise a new instance. Install the ordered collection that will 
	hold the runs."

	collection := OrderedCollection new.! !

!CompressedOrderedCollection methodsFor: 'converting'!

asOrderedCollection
	"Uncompress this collection."

	| newCollection |
	newCollection := OrderedCollection new.
	newCollection addAll: self.
	^newCollection! !

!CompressedOrderedCollection methodsFor: 'printing'!

storeOn: aStream 
	"Append to aStream an expression which, if evaluated, will generate   
	an object similar to the receiver."

	aStream nextPutAll: '(CompressedOrderedCollection new'.
	collection do: [:run | aStream nextPutAll: ' addRunWithLength: ' , run length
 printString , ' value: (' , run value storeString , ');'].
	aStream nextPutAll: ' yourself)'! !

!CompressedOrderedCollection class methodsFor: 'instance creation'!

new
	"Answer a new empty instance of OrderedCollection.

	Have to use basicNew because OrderedCollection re-defines new"

	^(self basicNew) initialize!

new: aNumber
	"Answer a new empty instance of OrderedCollection. This is here for compatibility,
 it totally ignores the parameter, since it is impossible to predict how many
 runs will be required.

	Have to use basicNew because OrderedCollection re-defines new"

	^self new! !

!OrderedCollection methodsFor: 'converting'!

asCompressedOrderedCollection
	"Convert this ordered collection into a compressed version of itself."

    |newCollection|
    newCollection := CompressedOrderedCollection new.
    newCollection addAll: self.
    ^ newCollection
! !

"COPYRIGHT.
 The above file is a Manchester Goodie protected by copyright.
 These conditions are imposed on the whole Goodie, and on any significant
 part of it which is separately transmitted or stored:
	* You must ensure that every copy includes this notice, and that
	  source and author(s) of the material are acknowledged.
	* These conditions must be imposed on anyone who receives a copy.
	* The material shall not be used for commercial gain without the prior
	  written consent of the author(s).
 Further information on the copyright conditions may be obtained by
 sending electronic mail:
	To: goodies-lib@cs.man.ac.uk
	Subject: copyright
 or by writing to The Smalltalk Goodies Library Manager, Dept of
 Computer Science, The University, Manchester M13 9PL, UK

 (C) Copyright 1993 University of Manchester
 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
	To: goodies-lib@cs.man.ac.uk
	Subject: help 
"!
