'From Objectworks(r)\Smalltalk, Release 4 of 25 October 1990 on 28 November 1991 at 4:05:48 pm'!

Dictionary subclass: #DictionaryWithDefault
        instanceVariableNames: 'defaultBlock '
        classVariableNames: ''
        poolDictionaries: ''
        category: 'Collections-Unordered'
!

DictionaryWithDefault comment:
'A DictionaryWithDefault is like a Dictionary, except that it is initialized
 with a value (or block) that is (yields) the default value for any key.

A subsequent access to a key that has not been explicitly inserted into the dictionary
 will yield the default value (calculated by executing the block), and insert
 the (key,default value) association into the dictionary.'
!

!DictionaryWithDefault methodsFor: 'copying'!

copyEmpty: aSize
        ^(super copyEmpty: aSize) defaultBlock: defaultBlock
! !

!DictionaryWithDefault methodsFor: 'accessing'!

associationAt: key 
        "Answer the association at key."

        ^super associationAt: key ifAbsent:
                [| value |
                value := defaultBlock value: key.
                super at: key put: value.
                super associationAt: key]
!

at: key 
        "Answer the value at key."

        ^super at: key ifAbsent:
                [| value |
                value := defaultBlock value: key.
                super at: key put: value.
                super at: key]
! !

!DictionaryWithDefault methodsFor: 'initialization'!

defaultBlock: aBlock
        defaultBlock := aBlock
! !

!DictionaryWithDefault class methodsFor: 'instance creation'!

newIdentity
        "Default map is identity."
        ^self newWithDefaultValueBlock: [ :x | x]
!

newWithDefaultValue: aValue
        ^super new defaultBlock: [ :ignored | aValue]
!

newWithDefaultValueBlock: aBlock
        ^super new defaultBlock: aBlock
! !

!DictionaryWithDefault class methodsFor: 'examples'!

example
        "DictionaryWithDefault example"
        "| d |
        d := DictionaryWithDefault example.
        d at: #foo put: 2.
        Transcript show: (d at: #foo) printString ; space ; show: (d at: #bar) printString"

        "A dictionary of symbols whose default value is the number of characters in
 the symbol."
        ^DictionaryWithDefault newWithDefaultValueBlock: [ :key | key size]!

example2
        "DictionaryWithDefault example2"
        "| d |
        d := DictionaryWithDefault example2.
        #(1 2 3 5 7 11 13 17 19) do: [ :n | d at: n put: 1].
        1 to: 20 do: [ :n | Transcript show: (d at: n) printString ; space]"
        "A dictionary of whose default value is 0."
        ^DictionaryWithDefault newWithDefaultValue: 0! !

"COPYRIGHT.
 The above file is a Manchester Goodie protected by copyright.
 These conditions are imposed on the whole Goodie, and on any significant
 part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).
 Further information on the copyright conditions may be obtained by
 sending electronic mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: copyright
 or by writing to The Smalltalk Goodies Library Manager, Dept of
 Computer Science, The University, Manchester M13 9PL, UK

 (C) Copyright 1992 University of Manchester
 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"!
