"       NAME            SparseArray
        AUTHOR          unknown
        CONTRIBUTOR     ParcBench
        FUNCTION        Implements sparse arrays
        ST-VERSION      2.5
        PREREQUISITES   
        CONFLICTS       
        DISTRIBUTION    world
        VERSION         1
        DATE     1990
        SUMMARY Sparse arrays (from ParcPlace Newsletter Summer 1990).
"

'Copyright (c) 1990, ParcPlace Systems, Inc.  All rights reserved.  The
contents of this file may be copied and used only in accord with the
provisions of the ParcBench User Agreement.

Claus: I dont know what this User Agreement says - 
the Manchester copyright below says that the code is free for non-commercial use,
so I included this in the goody collection ....'
!

'From Objectworks for Smalltalk-80(tm), Version 2.5 of 29 July 1989 on 23 May 1990 at 7:09:23 pm'!

ArrayedCollection subclass: #SparseArray
        instanceVariableNames: 'size defaultElement exceptionElements '
        classVariableNames: ''
        poolDictionaries: ''
        category: 'Collections-Indexed'
!

SparseArray comment:
'Instances of SparseArray behave like instances of Array in every respect
but have a very different representation.  Rather than storing each element
explicitly, SparseArrays have a default element and a list of exceptional
elements (and their respective indices).  This is an efficient representation

for arrays that are large and have very few elements that are not the
default.  Contrast this class with RunArray, which is optimized for the case

in which consecutive elements are likely to have the same value.

Instance Variables:
        size                    <SmallInteger> maximum number of elements
        defaultElement          <Object> the value of ''most'' elements
        exceptionElements       <IdentityDictionary of (SmallInteger->Object)>
                                list of the elements that are not the default, keyed
                                by index.
'!

!SparseArray methodsFor: 'accessing'!

at: index 
        "Answer the value of an indexable field in the receiver. Signal an error
        if the index is invalid."

        ^exceptionElements at: index ifAbsent:[
            self checkIndex: index.
            defaultElement
        ]
!

at: index put: value 
        "Store the argument value in the indexable field of the receiver indicated
        by index. Answer the value that was stored. Signal an error if the index
        is invalid."

        value = defaultElement ifTrue: [
            exceptionElements removeKey: index ifAbsent: []
        ] ifFalse:[
            self checkIndex: index.
            exceptionElements at: index put: value
        ].
        ^value
!

size
        "Answer the number of indexable fields in the receiver."

        ^size
! !

!SparseArray methodsFor: 'testing'!

includes: anObject 
        "Answer whether anObject is one of the receiver's elements."
        "Overrided for efficiency."

        anObject = defaultElement ifTrue: [^true].
        ^exceptionElements includes: anObject
!

occurrencesOf: anObject
        "Answer how many of the receiver's elements are equal to anObject."
        "Overrided for efficiency."

        anObject = defaultElement ifTrue: [^size - exceptionElements size].
        ^exceptionElements occurrencesOf: anObject
! !

!SparseArray methodsFor: 'adding'!

grow
        "The receiver becomes larger."
        "Growing simply means accepting larger subscripts."

        size := size + self growSize
!

growToAtLeast: anInteger
        "The receiver becomes at least as large as anInteger."
        "Growing simply means accepting larger subscripts."

        anInteger <= size ifTrue: [^self].
        anInteger class = SmallInteger ifFalse: [
            self error: 'Size must be non-negative SmallInteger.'
        ].
        size := anInteger
! !

!SparseArray methodsFor: 'copying'!

copyEmpty: aSize
        "Answer a copy of the receiver that contains no elements."

        ^self class new: aSize withAll: defaultElement
! !

!SparseArray methodsFor: 'private'!

checkIndex: index 
        "Signal an error if the argument is an invalid index, that is,
        less than 1 or greater than the array's size."

        index isInteger ifFalse: [^self nonIntegerIndexError: index].
        (index >= 1 and: [index <= size]) ifFalse: [
            ^self subscriptBoundsError: index
        ]
!

defaultElement

        ^defaultElement
!

setSize: anInteger default: default 
        "Initialize the receiver. Sent when creating new instances; answers the 
        receiver."
        "Indices must be SmallIntegers since we've chosen to use IdentityDictionary

        for the exceptionElements."

        (anInteger class = SmallInteger and: [anInteger >= 0]) ifFalse: [
            self error: 'Size must be non-negative SmallInteger.'
        ].
        size := anInteger.
        defaultElement := default.
        exceptionElements := IdentityDictionary new: 16
! !

!SparseArray class methodsFor: 'instance creation'!

new: size
        "Answer a new instance whose maximum subscript is size."

        ^self new: size withAll: nil
!

new: size withAll: anObject 
        "Answer a new instance whose every element is equal to the argument, 
        anObject."

        ^self basicNew setSize: size default: anObject
! !

" The above file is a Manchester Goodie.  It is distributed freely on condition
 that you observe these conditions in respect of the whole Goodie, and on
 any significant part of it which is separately transmitted or stored:
        * You must ensure that every copy includes this notice, and that
          source and author(s) of the material are acknowledged.
        * These conditions must be imposed on anyone who receives a copy.
        * The material shall not be used for commercial gain without the prior
          written consent of the author(s).

 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
        To: goodies-lib@cs.man.ac.uk
        Subject: help 
"!
