"       NAME            Cursor-fromString
	AUTHOR          Bernard Horan <bernard@is.morgan.com>
	CONTRIBUTOR     Bernard Horan <bernard@is.morgan.com>
	FUNCTION         Cursor instance creation method
	ST-VERSIONS     4.1
	PREREQUISITES    
	CONFLICTS        
	DISTRIBUTION    global
	VERSION         4.0
	DATE            September 1992
	SUMMARY         One method to create a Cursor from a (small)
string. Emphasis may also be supplied. Originally written by Pieter S.
van der Meulen for VI2.3, and suitably amended. BH, 25/9/92
Adapted to ST/X by claus"
!

'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 10 September 1992 at 3:05:04 am'!

!Cursor class methodsFor: 'string cursor examples'!

stringCursorExample1
	"example of a simple string-cursor"

	|v|

	v := View new realize.
	v cursor:(Cursor fromString:'hello' emphasis:#ignored)

	"Cursor stringCursorExample1"
!

stringCursorExample2
	"example of an animated string-cursor.
	 Needs the Cursor-Animated fileIn"

	|v|

	v := View new realize.
	(AnimatedCursor fromCursors:
	     (Array with:(Cursor fromString:'1')
		    with:(Cursor fromString:'2')
		    with:(Cursor fromString:'3')
		    with:(Cursor fromString:'4')
		    with:(Cursor fromString:'5'))) showIn:v

	"Cursor stringCursorExample2"
!

stringCursorExample3
	"example of an animated string-cursor
	 Needs the Cursor-Animated fileIn.
	 - you could use this for your application"

	|v|

	v := View new realize.
	(AnimatedCursor fromCursors:
	     (Array with:(Cursor fromString:'busy .')
		    with:(Cursor fromString:'busy ..')
		    with:(Cursor fromString:'busy ...')
		    with:(Cursor fromString:'busy ....')
		    with:(Cursor fromString:'busy .....'))) showIn:v

	"Cursor stringCursorExample3"
!

stringCursorExample4
	"example of an animated string-cursor
	 Needs the Cursor-Animated and Form-misc fileIns.
	 - even better, computes a mask for the cursor"

	|v|

	v := View new realize.
	(AnimatedCursor fromCursors:
	     (Array with:(Cursor withMaskFromString:'busy .')
		    with:(Cursor withMaskFromString:'busy ..')
		    with:(Cursor withMaskFromString:'busy ...')
		    with:(Cursor withMaskFromString:'busy ....')
		    with:(Cursor withMaskFromString:'busy .....'))) showIn:v

	"Cursor stringCursorExample4"
! !

!Cursor class methodsFor: 'instance creation'!

fromString: aString  emphasis:aSymbol 
	"Create a Cursor specified by aString. 
	Be sure to keep aString small, e.g. 'S' or '2'. 
	The emphasis is specified by aSymbol but is ignored. 
	Originally written by Pieter S. van der Meulen for VI2.3 
	and subsequently amended by Bernard Horan, 10 September 1992.
	and finally brought to ST/X feb 93 by claus."

	^ self fromString:aString
!

fromString: aString
	"Create a Cursor specified by aString. 
	Be sure to keep aString small, e.g. 'S' or '2'. 
	The emphasis is specified by aSymbol. 
	Originally written by Pieter S. van der Meulen for VI2.3 
	and subsequently amended by Bernard Horan, 10 September 1992.
	and finally brought to ST/X feb 93 by claus."

	| font form |

	font := DisplayRootView new font. "to get the default font"
	font := font on:Display.
	form := Form extent:(font widthOf:aString) @ font height.
	form clear.
	form displayString:aString x:0 y:(font ascent).
	^ self sourceForm:form maskForm:form hotSpot:(0@0)
!

withMaskFromString: aString
	"this one needs the Form-misc filein to smear a form.
	and finally brought to ST/X feb 93 by claus."

	| font form mask |

	font := DisplayRootView new font. "to get the default font"
	font := font on:Display.
	form := Form extent:(font widthOf:aString) @ font height.
	form clear.
	form displayString:aString x:0 y:(font ascent).
	mask := form smeared.
	^ self sourceForm:form maskForm:mask hotSpot:(0@0)
! !

"COPYRIGHT.
 The above file is a Manchester Goodie protected by copyright.
 These conditions are imposed on the whole Goodie, and on any significant
 part of it which is separately transmitted or stored:
	* You must ensure that every copy includes this notice, and that
	  source and author(s) of the material are acknowledged.
	* These conditions must be imposed on anyone who receives a copy.
	* The material shall not be used for commercial gain without the prior
	  written consent of the author(s).
 Further information on the copyright conditions may be obtained by
 sending electronic mail:
	To: goodies-lib@cs.man.ac.uk
	Subject: copyright
 or by writing to The Smalltalk Goodies Library Manager, Dept of
 Computer Science, The University, Manchester M13 9PL, UK

 (C) Copyright 1993 University of Manchester
 For more information about the Manchester Goodies Library (from which 
 this file was distributed) send e-mail:
	To: goodies-lib@cs.man.ac.uk
	Subject: help 
"!
