"
 this example comes from:

 Jan Steinman, Bytesmiths
 2002 Parkside Court, West Linn, OR 97068-2767 USA, +1 503 657 7703
 Friedlistrasse 19, CH-3006, Bern, Switzerland, +41 31 999 3946

 Claus:
   this was published in comp.lang.smalltalk; added here as an example ...
"

'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 
ENVY/Developer R1.41 of 1 February 1993 on 16 June 1993 at 3:12:32 pm'!

Fraction subclass: #Fixed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category:'Magnitude-Numbers'
!

Fixed comment: '
Description: This class implements infinite precision fixed-point numbers. 
It doesn''t really do anything too interesting except creating instances, converting, and printing, 
since its superclass Fraction does all the work.

Test: "''123456789012345678901234567890.123456789'' asFixed * 1000000000 = 123456789012345678901234567890123456789"

Notes: 1) The current implementation does not convert arbitrarily-based String representations, 
	  which shouldn''t be too much a problem for financial types.'!

!Fixed publicMethodsFor: 'accessing'!

places
	"Return the number of places of significance that is carried by the receiver."

	^self denominator log! !

!Fixed publicMethodsFor: 'printing'!

printString
	"return a printed representation of the fixed"

	^ (numerator // denominator) printString , '.' ,
	  ((numerator \\ denominator) printStringZeroPaddedTo: self places)
!

printOn: aStream 
	"Append to the argument, aStream, the receiver's numerator divided by its denominator."

	(numerator // denominator) printOn: aStream.
	aStream nextPut: $..
	(numerator \\ denominator) printOn: aStream zeroPaddedTo: self places
! !

!Fixed publicMethodsFor: 'private'!

setNumerator: nInteger places: places 
	"Initialize the instance variables. Fail if the denominator is zero."

	self setNumerator: nInteger denominator: (10 raisedTo: places)
! !

!Fixed class publicMethodsFor: 'instance creation'!

fromString: fixedPointString
	"Answer a new instance of the receiver which is an interpretation of the digits found in <fixedPointString>."

	^self readFrom: (ReadStream on: fixedPointString)
!

readFrom: aStream
	"Answer an instance of me as described on the stream, aStream."

	| integerPart fractionStream char fractionPart places |
	(aStream atEnd or: [aStream peek isLetter]) ifTrue: [^0].

	integerPart := (aStream upTo: $.) asNumber.
	(aStream atEnd or: [aStream peek isLetter]) ifTrue: [^integerPart].

	fractionStream := ReadWriteStream on: (String new: 10).
		[char := aStream next.
		char ~~ nil and: [char isDigit]] whileTrue: [fractionStream nextPut: char].
	places := fractionStream contents size.
"
	fractionPart := Number readFrom: fractionStream reset.
"
	fractionPart := Number readFromString:(fractionStream contents).
	^self basicNew setNumerator: integerPart * (10 raisedTo: places) + fractionPart places: places
! !

!String methodsFor:'converting'!

asFixed
    "return a new Fixed from the printed description in the receiver"

    ^ Fixed readFrom:(ReadStream on:self)
! !
