"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

ByteArray subclass:#Structure
       instanceVariableNames:''
       classVariableNames:''
       poolDictionaries:''
       category:'System-Support'
!

Structure class instanceVariableNames:'FieldDefinition'!

Structure comment:'

COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

$Header: /claus/smalltalk/st/goodies/RCS/Structure.st,v 1.2 95/03/09 02:18:28 claus Exp $

'!

!Structure class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    NOTICE: there could be much more here ...

    Structures are helpful, when interfaces to external programs
    (usually c-programs), files or databases are needed.
    ((Actually, the motivation to create this class came from the need 
      to interface to a C-program via a Socket ...))

    Instances can store and retrieve data in a layout compatible 
    to those interfaces by performing an automatic conversion between
    objects and the external representation.

    Structure defines a new set of definition messages to create 
    structure classes. Definition of a structure automatically
    creates corresponding access methods.

    Structure definition:

	 Structure name:<name>
		 fields:#(
			     ( <name> <type> <size> [<options>] )
					    ...
			 )

    the following types are currently implemented:

	    Integer         will be stored/retrieved as SmallInteger or
			    LargeInteger depending on value

	    String          will store/retrieve strings
			    options: #withZeroByte
				     (fill: fillCharacter)

	    <structure>     a substructure

    padding is currently not implemented; other types may follow on demand.

    TODO: parse c include files and create Structure automatically.


    example:

	 to interface to a c-program and pass it a structure as:

	    struct ipcmessage {
		short       type;
		int         timeStamp;
		char        message[10];
	    };

	 the following structure will fit:

	    Structure name:#IPCMessage
		    fields:#(
				('type'             Integer 2          )
				('timeStamp'        Integer 4          )
				('message'          String  10         )
			    )

	 such a structure is created with:

	    aMessage := IPCMessage new

	 the fields can be accessed with:

	    aMessage type      -> returns type-field
	    aMessage type:val  -> to set type field

	 access methods for all fields are defined automatically.
	 Since the structure is really a ByteArray-like Object, it can be 
	 stored/retrieved in a C-compatible format using:

	 writing:
	    aStream binary.
	    aStream nextPutAll:aMessage

	 reading:
	    aMessage := someStructureClass new.
	    aStream binary.
	    aStream nextBytes:(aMessage basicSize) into:aMessage

	 A nice trick, if you dont know, what type of message will arrive:

	    buffer := ByteArray new:maxSize.
	    aStream binary.
	    aStream nextBytes:maxSize into:buffer.
	    <determine type from some byte-fields ...>
	    buffer changeClassTo:<whatEverStructure you think it is>


    more examples:

	Structure name:#MessageHeader
		fields:#(
			    ('type'             Integer    2                  )
			    ('timeStamp'        Integer    4                  )
			    ('message'          String     10 (#withZeroByte) )
			)

	Structure name:#Message1
		fields:#(
			    ('header'           MessageHeader      )
			    ('largeVal'         Integer        8   )
			)

	Structure name:#Message2
		fields:#(
			    ('header'           MessageHeader      )
			    ('short1'           Integer        2   )
			    ('short2'           Integer        2   )
			)

	Structure name:#Message3
		fields:#(
			    ('header'           MessageHeader      )
			    ('floatValue'       Float              )
			)

	Structure name:#Message4
		fields:#(
			    ('header'           MessageHeader      )
			    ('doubleValue'      Double             )
			)

	MessageHeader new
	Message1 new
	Message2 new
	Message3 new
	Message4 new


	|m|
	m := Message4 new.
	m type:1.
	m timeStamp:12345.
	m message:'hello'.
	m doubleValue:3.14159.
	m inspect


    storing in a C language compatible format:

	|m s|
	m := Message4 new.
	m type:1.
	m timeStamp:12345.
	m message:'hello'.
	m doubleValue:3.14159.
	s := 'testfile' asFilename writeStream binary.
	s nextPutAll:m.
	s close


	|s m|
	s := 'testfile' asFilename readStream binary.
	m := s nextAvailable:10000.
	m := changeClassTo:Message4.
	m inspect
        
"
! !

!Structure class methodsFor:'definition'!

name:name fields:fieldSpec
    "to define a structure with no category, no classVariables"

    ^ self name:name fields:fieldSpec classVariableNames:'' category:'structures' 
!

name:name fields:fieldSpec category:cat
    "to define a structure with no classVariables"

    ^ self name:name fields:fieldSpec classVariableNames:'' category:cat 
!

name:name fields:fieldSpec classVariableNames:classVars category:cat 
    "define a structure named name, which must be a symbol. 
     The fields argument is an array where each element defines one element of the structure.
     Each fields element describes one field of the structure, it must be an array containing name, 
     type and size of the field.
     syntax:

	Structure name:<name>
		fields:#(
			    ( <name> <type> <size> [<options>] )
					...
			)
     example:

	Structure name:#MessageHeader
		fields:#(
			    ('type'             Integer    2                  )
			    ('timeStamp'        Integer    4                  )
			    ('message'          String     10 (#withZeroByte) )
			)

	Structure name:#Message1
		fields:#(
			    ('header'           MessageHeader      )
			    ('largeVal'         Integer        8   )
			)

	Structure name:#Message2
		fields:#(
			    ('header'           MessageHeader      )
			    ('short1'           Integer        2   )
			    ('short2'           Integer        2   )
			)

	MessageHeader new
	Message1 new
	Message2 new

    the following types are currently implemented:

	Integer         will be stored/retrieved as SmallInteger or
			LargeInteger depending on value

	String          will store/retrieve strings
			optional (#withZeroByte)
			optional (#fill: fillCharacter)
    "

    |newClass upd|

    upd := Class updateChanges:false.

    ((Smalltalk includesKey:name) and:[(Smalltalk at:name) notNil]) ifTrue:[
	Smalltalk removeClass:(Smalltalk at:name)
    ].

    newClass := self subclass:name 
		     instanceVariableNames:''
		     classVariableNames:classVars
		     poolDictionaries:''
		     category:cat.
    newClass fieldDefinition:fieldSpec.

    Class updateChanges:upd.
    ^ newClass
! !

!Structure class methodsFor:'constants'!

structSize
    self subclassresponsibility
! !

!Structure class methodsFor:'instance creation'!

new
    ^ self basicNew:(self structSize)
! !

!Structure class methodsFor:'queries'!

typeOf:aFieldSymbol
    FieldDefinition do:[:field |
	|name type size|

	name := field at:1.
	name == aFieldSymbol ifTrue:[
	    type := field at:2.

	    (#(Integer String) includes:type) ifTrue:[
		size := field at:3.
	    ] ifFalse:[
		type == #Float ifTrue:[
		    size := 4
		] ifFalse:[
		    type == #Double ifTrue:[
			size := 8
		    ] ifFalse:[
			((Smalltalk includesKey:type) and:[(Smalltalk at:type) isKindOf:Structure class]) ifTrue:[
			    size := (Smalltalk at:type) structSize
			] ifFalse:[
			    size := 0
			].
		    ]
		]
	    ].
	    ^ type
	]
    ].
    ^ nil
! !

!Structure class methodsFor:'private'!

fieldDefinition
    ^ FieldDefinition
!

fieldDefinition:aDefinition
    |offset|

    FieldDefinition := aDefinition.

    "create methods to access things"

    offset := 1.
    aDefinition do:[:field |
	offset := self createAccessMethodsFor:field offset:offset.
    ].

    "create instance creation method"

    Compiler compile:'structSize ^ ' , (offset - 1) printString
	    forClass:self class inCategory:'constants'.
!

createAccessMethodsFor:fieldDef offset:offset
    |type size options defs sel o|

    defs := fieldDef.
    type := defs at:2.
    (#(Integer String) includes:type) ifTrue:[
	sel := ('create' , type , 'FieldNamed:type:size:options:offset:') asSymbol.
	size := defs at:3.
    ] ifFalse:[
	((type == #Float) or:[type == #Double]) ifTrue:[
	    sel := #'createFloatFieldNamed:type:options:offset:'.
	    type == #Double ifTrue:[
		size := 8
	    ] ifFalse:[
		size := 4
	    ]
	] ifFalse:[
	    ((Smalltalk includesKey:type) 
	    and:[(Smalltalk at:type) isKindOf:Structure class]) ifFalse:[
		self error:'unknown field type: ', type.
		^ self
	    ].
	    o := offset.
	    (Smalltalk at:type) fieldDefinition do:[:subField |
		o := self createAccessMethodsFor:subField offset:o.
	    ].
	    ^ o
	]
    ].

    defs size < 4 ifTrue:[
	defs := defs copyWith:nil
    ] ifFalse:[
	fieldDef size > 4 ifTrue:[
	    self error:'bad field definition'.
	    ^ self
	].
    ].
    defs := defs copyWith:offset.
    self perform:sel withArguments:defs.
    ^ offset + size
!

createIntegerFieldNamed:name type:type size:size options:options offset:offset 
    |getCode setCode|

    size == 1 ifTrue:[
	getCode := name , ' ^ self basicAt:' , offset printString.
	setCode := name , ':val self basicAt:' , offset printString
			, ' put:val. ^ val'.
    ] ifFalse:[
	size == 2 ifTrue:[
	    getCode := name , ' ^ self wordAt:' , offset printString.
	    setCode := name , ':val self wordAt:' , offset printString
			    , ' put:val. ^ val'.
	] ifFalse:[
	    size == 4 ifTrue:[
		getCode := name , ' ^ self doubleWordAt:' , offset printString.
		setCode := name , ':val self doubleWordAt:' , offset printString
				, ' put:val. ^ val'.
	    ] ifFalse:[
		getCode := name , ' ^ self integerFieldAt:' , offset printString
				, ' size:' , size printString.
		setCode := name , ':val self integerFieldAt:' , offset printString
				, ' put:val size:' , size printString , '. ^ val'.
	    ]
	].
    ].
    Compiler compile:getCode forClass:self inCategory:'accessing'.
    Compiler compile:setCode forClass:self inCategory:'accessing'.
!

createFloatFieldNamed:name type:type options:options offset:offset 
    |getCode setCode|

    type == #Float ifTrue:[
	getCode := name , ' ^ self ieeFloatAt:' , offset printString.
	setCode := name , ':val self ieeFloatAt:' , offset printString
			, ' put:val. ^ val'.
    ] ifFalse:[
	type == #Double ifTrue:[
	    getCode := name , ' ^ self ieeDoubleAt:' , offset printString.
	    setCode := name , ':val self ieeDoubleAt:' , offset printString
			    , ' put:val. ^ val'.
	].
    ].
    Compiler compile:getCode forClass:self inCategory:'accessing'.
    Compiler compile:setCode forClass:self inCategory:'accessing'.
!

createStringFieldNamed:name type:type size:size options:options offset:offset 
    |getCode setCode|

    (options notNil and:[options includes:#withZeroByte]) ifTrue:[
	getCode := name , ' ^ self stringFieldAt:' , offset printString
			, ' size:' , size printString.
	setCode := name , ':val self string0FieldAt:' , offset printString
			, ' put:val size:' , size printString , '. ^ val'
    ] ifFalse:[
	getCode := name , ' ^ self stringFieldAt:' , offset printString
			, ' size:' , size printString.
	setCode := name , ':val self stringFieldAt:' , offset printString
			, ' put:val size:' , size printString , '. ^ val'
    ].
    Compiler compile:getCode forClass:self inCategory:'accessing'.
    Compiler compile:setCode forClass:self inCategory:'accessing'.
! !

!Structure class methodsFor:'fileout'!

fileOutDefinitionOn:aStream
    "print an expression to define myself on aStream"

    self == Structure ifTrue:[^ super fileOutDefinitionOn:aStream].

    aStream nextPutAll:'Structure name:#'.
    aStream nextPutAll:self name asString.
    aStream cr; tab.
    aStream nextPutAll:'fields:#('.
    aStream cr.
    FieldDefinition do:[:aFieldDef |
	aStream tab; tab.
	aStream nextPutAll:'( '.
	aFieldDef do:[:spec |
	    (spec isKindOf:Array) ifTrue:[
		aStream nextPutAll:'('.
		spec do:[:el |
		    aStream nextPutAll:el printString.
		    aStream space
		].
		aStream nextPutAll:')'.

	    ] ifFalse:[
		aStream nextPutAll:(spec printStringPaddedTo:15 with:(Character space)).
	    ].
	    aStream tab.
	].
	aStream nextPutAll:' )'.
	aStream cr.
    ].
    aStream cr; tab; nextPutAll:')'; cr.
    aStream tab; nextPutAll:'classVariableNames:'''.
    aStream nextPutAll:self classVariableString.
    aStream nextPutAll:''''.
    aStream cr; tab; nextPutAll:'category:'''.
    aStream nextPutAll:(self category printString).
    aStream nextPutAll:''''.
!

fileOutOn:aStream
    "file out all methods onto aStream"

    |collectionOfCategories|

    self == Structure ifTrue:[^ super fileOutOn:aStream].

    aStream nextPutAll:(Smalltalk timeStamp).
    aStream nextPut:$!!. 
    aStream cr.
    aStream cr.
    self fileOutDefinitionOn:aStream.
    aStream nextPut:$!!. 
    aStream cr.
    aStream cr.
    self class instanceVariableString isBlank ifFalse:[
	self fileOutClassInstVarDefinitionOn:aStream.
	aStream nextPut:$!!. 
	aStream cr.
	aStream cr
    ].

    comment notNil ifTrue:[
	aStream nextPutAll:name.
	aStream nextPutAll:' comment:'.
	aStream nextPutAll:(comment storeString).
	aStream nextPut:$!!.
	aStream cr.
	aStream cr
    ].
    (self class implements:#initialize) ifTrue:[
	aStream nextPutAll:(name , ' initialize').
	aStream nextPut:$!!. 
	aStream cr
    ]
! !

!Structure methodsFor:'error handling'!

fieldTooShortError
    self error:'field is too short to store this object'
! !

!Structure methodsFor:'queries'!

typeOf:aFieldSymbol
    ^ self class typeOf:aFieldSymbol
! !

!Structure methodsFor:'private accessing'!

integerFieldAt:offset size:size
    |val o|

    val := 0.
    o := offset "+ size - 1".
    1 to:size do:[:i |
	val := (val * 256) + (self at:o).
	o := o "- 1" + 1
    ].
    ^ val
!

integerFieldAt:offset put:val size:size
    |v o|

    v := val.
    o := offset + size - 1.
    1 to:size do:[:i |
	self at:o put:(v \\ 256).
	v := v // 256.
	o := o "+ 1" - 1
    ].
    ^ val
!

stringFieldAt:offset size:size
    |bytes str o|

    str := String new:size.
    o := offset.
    1 to:size do:[:i |
	|v|

	v := self at:o.
	v == 0 ifTrue:[
	    ^ str copyFrom:1 to:(i-1)
	].
	str at:i put:(Character value:v).
	o := o + 1
    ].
    ^ str
!

stringFieldAt:offset put:str size:size pad:padChar
    |o end l|

    o := offset.
    l := str size.
    l > size ifTrue:[
	self fieldTooShortError
    ].
    end := l min:size.
    1 to:end do:[:i |
	self at:o put:(str at:i) asciiValue.
	o := o + 1.
    ].
    end+1 to:size do:[:i |
	self at:o put:padChar asciiValue.
	o := o + 1.
    ].
    ^ str
!

stringFieldAt:offset put:str size:size
    ^ self stringFieldAt:offset put:str size:size pad:(Character space)
!

string0FieldAt:offset put:str size:size
    |s|

    s := str.
    s size >= size ifTrue:[
	self fieldTooShortError.
	s := s copyFrom:1 to:(size - 1).
    ].
    ^ self stringFieldAt:offset put:s size:size pad:(Character value:0)
! !

!Structure methodsFor:'inspecting'!

inspectorClass
    ^ StructureInspectorView
! !

!Structure methodsFor:'storing'!

storeOn:aStream
    aStream nextPutBytes:(self size) from:self startingAt:1
! !

!Structure class methodsFor:'printing support'!

addFieldStringsOf:aStruct from:aDefinition to:aString performing:printSelector first:f
    |s first|

    s := aString.
    first := f.
    aDefinition do:[:fieldDef |
	|name type|

	first ifTrue:[
	    first := false
	] ifFalse:[
	    s := s , ' '
	].
	name := fieldDef at:1.
	type := fieldDef at:2.
	((Smalltalk includesKey:type) 
	and:[(Smalltalk at:type) isKindOf:Structure class]) ifTrue:[
	    s := (Smalltalk at:type) addFieldStringsOf:aStruct from:(Smalltalk at:type) fieldDefinition
						    to:s performing:printSelector first:first
	] ifFalse:[
	    s := s , name , '->' , ((aStruct perform:name asSymbol) perform:printSelector)
	]
    ].
    ^ s.
! !

!Structure methodsFor:'printing'!

displayString
    |s|

    s := self class name , '('.
    s := self class addFieldStringsOf:self from:self class fieldDefinition to:s performing:#displayString first:true.
    ^ s , ')'
!

printString
    |s|

    s := self class name , '('.
    s := self class addFieldStringsOf:self from:self class fieldDefinition to:s performing:#printString first:true.
    ^ s , ')'
! !
